import logging

from dirty_equals import IsNow, IsPositiveInt

from foxglove import BaseSettings
from foxglove.db.migrations import run_migrations
from foxglove.db.patches import Patch, run_patch
from foxglove.db.utils import AsyncPgContext
from tests.conftest import SyncConnContext


def test_patch_live(settings: BaseSettings, wipe_db, caplog, loop):
    caplog.set_level(logging.INFO, 'foxglove.db')
    run_patch('insert_org', True, {})
    with SyncConnContext(settings.pg_dsn, loop) as conn:
        assert conn.fetchval('select count(*) from organisations') == 1

    assert caplog.messages == [
        '--------- running patch insert_org live ----------',
        '------------- live, committed patch --------------',
    ]


def test_patch_dry_run(settings: BaseSettings, wipe_db, caplog, loop):
    caplog.set_level(logging.INFO)
    run_patch('insert_org', False, {})
    with SyncConnContext(settings.pg_dsn, loop) as conn:
        assert conn.fetchval('select count(*) from organisations') == 0

    assert caplog.messages == [
        '------- running patch insert_org not live --------',
        '------------- not live, rolling back -------------',
    ]


def test_patch_error(settings: BaseSettings, wipe_db, caplog, loop):
    caplog.set_level(logging.INFO, 'foxglove.db')
    run_patch('insert_org', False, {'fail': '1'})
    with SyncConnContext(settings.pg_dsn, loop) as conn:
        assert conn.fetchval('select count(*) from organisations') == 0

    assert caplog.messages == [
        '------- running patch insert_org not live --------',
        '--------------------- error ----------------------',
        'Error running insert_org patch',
    ]


async def test_run_migrations_ok(settings: BaseSettings, wipe_db, db_conn, caplog):
    async def ok_patch(logger, **kwargs):
        logger.info('running ok_patch')

    patches = [Patch(ok_patch, auto_run='foobar')]

    caplog.set_level(logging.DEBUG, 'foxglove.db')
    assert await run_migrations(settings, patches, True) == 1
    async with AsyncPgContext(settings.pg_dsn) as conn:
        assert await conn.fetchval("select exists (select from pg_tables where tablename='migrations')") is True
        assert await conn.fetchval('select count(*) from migrations') == 1
        migrations = dict(await conn.fetchrow('select * from migrations'))

    assert migrations == {
        'id': IsPositiveInt,
        'ref': 'ok_patch:foobar',
        'sql_section': '-',
        'ts': IsNow(tz='utc'),
        'fake': False,
    }
    assert await run_migrations(settings, patches, True) == 0

    async with AsyncPgContext(settings.pg_dsn) as conn:
        assert await conn.fetchval('select count(*) from migrations') == 1

    async with AsyncPgContext(settings.pg_dsn) as conn:
        async with conn.transaction():
            await conn.execute('lock table migrations')
            assert await run_migrations(settings, patches, True) == 0

    assert caplog.messages == [
        'migrations table created',
        'checking 1 migration patches...',
        '-------------- ok_patch:foobar ... ---------------',
        'running ok_patch',
        '--------------- ok_patch:foobar ✓ ----------------',
        '1 migration patches run, 0 already up to date ✓',
        'checking 1 migration patches...',
        'all 1 migrations already up to date ✓',
        'another transaction has locked migrations, skipping migrations here',
    ]


async def test_run_migrations_error(settings: BaseSettings, wipe_db, caplog):
    def ok_patch(logger, **kwargs):
        return 'hello'

    async def error_patch(**kwargs):
        raise ValueError('broken')

    patches = [Patch(ok_patch, auto_run=True), Patch(error_patch, auto_run=True)]

    caplog.set_level(logging.INFO, 'foxglove.db')
    assert await run_migrations(settings, patches, True) == 0

    async with AsyncPgContext(settings.pg_dsn) as conn:
        assert await conn.fetchval("select exists (select from pg_tables where tablename='migrations')") is False

    assert caplog.messages == [
        'migrations table created',
        'checking 2 migration patches...',
        '------------------ ok_patch ... ------------------',
        'result: hello',
        '------------------- ok_patch ✓ -------------------',
        '---------------- error_patch ... -----------------',
        '--------------- error_patch failed ---------------',
        'Error running error_patch migration patch',
        'patch failed, rolling back all 1 migration patches in this session',
    ]


async def test_run_migrations_none(settings: BaseSettings, wipe_db, caplog):
    caplog.set_level(logging.INFO, 'foxglove.db')
    assert await run_migrations(settings, [], True) == 0

    async with AsyncPgContext(settings.pg_dsn) as conn:
        assert await conn.fetchval("select exists (select from pg_tables where tablename='migrations')") is False

    assert caplog.messages == []


async def test_run_migrations_not_live(settings: BaseSettings, wipe_db, db_conn, caplog):
    async def ok_patch(logger, **kwargs):
        logger.info('running ok_patch')

    patches = [Patch(ok_patch, auto_run=True)]

    caplog.set_level(logging.DEBUG, 'foxglove.db')
    assert await run_migrations(settings, patches, False) == 1

    async with AsyncPgContext(settings.pg_dsn) as conn:
        assert await conn.fetchval("select exists (select from pg_tables where tablename='migrations')") is False

    assert caplog.messages == [
        'migrations table created',
        'checking 1 migration patches...',
        '------------------ ok_patch ... ------------------',
        'running ok_patch',
        '------------------- ok_patch ✓ -------------------',
        '1 migration patches run, 0 already up to date, not live rolling back',
    ]


async def test_run_migrations_fake(settings: BaseSettings, wipe_db, db_conn, caplog):
    async def ok_patch(logger, **kwargs):
        raise RuntimeError('should not happen')

    patches = [Patch(ok_patch, auto_run=True)]

    caplog.set_level(logging.DEBUG, 'foxglove.db')
    assert await run_migrations(settings, patches, True, fake=True) == 1

    async with AsyncPgContext(settings.pg_dsn) as conn:
        assert await conn.fetchval("select exists (select from pg_tables where tablename='migrations')") is True
        assert await conn.fetchval('select count(*) from migrations') == 1
        migrations = dict(await conn.fetchrow('select * from migrations'))

    assert migrations == {
        'id': IsPositiveInt,
        'ref': 'ok_patch',
        'sql_section': '-',
        'ts': IsNow(tz='utc'),
        'fake': True,
    }

    assert caplog.messages == [
        'migrations table created',
        'checking 1 migration patches...',
        'faked migration ok_patch',
        '1 migration patches faked, 0 already up to date ✓',
    ]



from pathlib import Path
from typing import List, Optional

from arq.connections import RedisSettings

from foxglove.settings import BaseSettings

THIS_DIR = Path(__file__).parent.resolve()


class Settings(BaseSettings):
    sql_path: Path = THIS_DIR / 'models.sql'
    patch_paths: List[str] = ['demo.patches']
    pg_dsn = 'postgres://postgres@localhost:5432/foxglove_demo'
    app = 'demo.main:app'
    redis_settings: Optional[RedisSettings] = 'redis://localhost:6379/6'
    template_dir = THIS_DIR / 'templates'
    static = THIS_DIR / 'static'

    worker_func = 'demo.main:worker'



import pytest
from httpx import Request as HttpxRequest, Response as HttpxResponse

from foxglove.exceptions import UnexpectedResponse


def test_unexpected_response_ok(settings):
    UnexpectedResponse.check(HttpxResponse(200, request=HttpxRequest('GET', url='https://example.com')))


def test_unexpected_response_error(settings):
    r = HttpxResponse(403, request=HttpxRequest('GET', url='https://example.com'), text='{"foo": 1}')
    with pytest.raises(UnexpectedResponse) as exc_info:
        UnexpectedResponse.check(r)

    assert repr(exc_info.value) == (
        'UnexpectedResponse("GET https://example.com, unexpected response: 403:\n{\n  "foo": 1\n}")'
    )



import asyncio
import os

import pytest
from buildpg import asyncpg
from starlette.requests import Request
from starlette.responses import Response

from demo.settings import Settings
from foxglove import glove
from foxglove.db import lenient_conn, prepare_database
from foxglove.db.helpers import DummyPgPool, SyncDb
from foxglove.test_server import create_dummy_server
from foxglove.testing import TestClient

commit_transactions = 'KEEP_DB' in os.environ


@pytest.fixture(scope='session', name='settings')
def fix_settings():
    settings = Settings(dev_mode=False, test_mode=True, bcrypt_rounds=4)
    assert not settings.dev_mode
    glove._settings = settings

    yield settings

    glove._settings = None


@pytest.fixture(scope='session', name='alt_settings')
def fix_alt_settings(settings: Settings):
    return settings.copy(update=dict(pg_dsn='postgres://postgres@localhost:5432/foxglove_demo_alt'))


@pytest.fixture(name='db_conn_global')
async def _fix_db_conn_global(settings):
    conn = await lenient_conn(settings, with_db=False, sleep=0)

    yield conn

    await conn.close()


@pytest.fixture(name='loop')
def fix_loop(event_loop):
    asyncio.set_event_loop(event_loop)
    return event_loop


@pytest.fixture(scope='session', name='clean_db')
def fix_clean_db(settings):
    asyncio.run(prepare_database(settings, True, run_migrations=False))


@pytest.fixture(name='wipe_db')
async def fix_wipe_db(settings):
    await prepare_database(settings, True, run_migrations=False)
    yield
    await prepare_database(settings, True, run_migrations=False)


@pytest.fixture(name='db_conn')
async def fix_db_conn(settings, clean_db):
    conn = await asyncpg.connect_b(dsn=settings.pg_dsn)

    tr = conn.transaction()
    await tr.start()

    yield DummyPgPool(conn)

    if commit_transactions:
        await tr.commit()
    else:
        await tr.rollback()
    await conn.close()


@pytest.fixture(name='glove')
async def fix_glove(db_conn, loop):
    glove.pg = db_conn

    await glove.startup()
    await glove.redis.flushdb()

    yield glove

    await glove.shutdown()


@pytest.fixture(name='sync_db')
def fix_sync_db(db_conn, loop):
    return SyncDb(db_conn, loop)


class ConnContext:
    def __init__(self, dsn):
        self._dsn = dsn
        self._conn = None

    async def __aenter__(self) -> asyncpg.Connection:
        self._conn = await asyncpg.connect(self._dsn)
        return self._conn

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self._conn.close()


class SyncConnContext:
    def __init__(self, dsn, loop=None):
        self._dsn = dsn
        self._conn = None
        self._loop = loop or asyncio.get_event_loop()

    def __enter__(self) -> SyncDb:
        self._conn = self._loop.run_until_complete(asyncpg.connect(self._dsn))
        return SyncDb(self._conn, self._loop)

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._loop.run_until_complete(self._conn.close())


@pytest.fixture(name='client')
def fix_client(settings: Settings, db_conn, glove, loop):
    app = settings.create_app()
    with TestClient(app, loop=loop) as client:
        yield client


@pytest.fixture(name='client_sentry')
def fix_client_sentry(settings: Settings, db_conn, glove, loop):
    settings.sentry_dsn = 'https://123@example.com/789'
    glove._settings = settings
    app = settings.create_app()
    with TestClient(app, loop=loop) as client:
        yield client
    settings.sentry_dsn = None


@pytest.fixture(name='dummy_server')
async def _fix_dummy_server(loop):
    ds = await create_dummy_server(loop)
    yield ds
    await ds.stop()


@pytest.fixture(name='create_request')
def _fix_create_request(settings: Settings):
    app = settings.create_app()

    async def endpoint(request: Request):
        return Response()

    class CreateRequest:
        def __init__(self, _app):
            self.app = _app

        def __call__(self, method='GET', path='/', headers=None, client_addr='testclient', **extra_scope):
            _headers = {'host': 'testserver', 'user-agent': 'testclient', 'connection': 'keep-alive'}
            if headers:
                _headers.update(headers)
            scope = {
                'type': 'http',
                'http_version': '1.1',
                'method': method,
                'path': path,
                'root_path': '',
                'scheme': 'http',
                'query_string': b'',
                'headers': [(k.lower().encode(), v.encode()) for k, v in _headers.items()],
                'client': [
                    client_addr,
                    50000,
                ],
                'server': [
                    'testserver',
                    80,
                ],
                'extensions': {
                    'http.response.template': {},
                },
                'app': app,
                'state': {},
                'session': {},
                'endpoint': endpoint,
                'path_params': {},
                **extra_scope,
            }
            return Request(scope, None, None)

    return CreateRequest(app)



import json

import pytest
from starlette.requests import Request
from starlette.responses import Response

import foxglove.middleware
from foxglove.middleware import CloudflareCheckMiddleware, CsrfMiddleware, HostRedirectMiddleware
from foxglove.testing import TestClient as Client


async def next_function(request: Request):
    return Response('ok')


async def test_host_redirect_ok(create_request):
    req: Request = create_request(headers={'host': 'good'})
    m = HostRedirectMiddleware(create_request.app, 'good')
    r = await m.dispatch(req, next_function)
    assert r.status_code == 200, r.body
    assert r.headers.get('location') is None
    assert r.body == b'ok'


async def test_host_redirect_wrong(create_request):
    req: Request = create_request(headers={'host': 'bad'})
    m = HostRedirectMiddleware(create_request.app, 'good')
    r = await m.dispatch(req, next_function)
    assert r.status_code == 301, r.body
    assert r.headers.get('location') == 'http://good/'
    assert r.body == b''


async def test_cloudflare_ok_header(create_request, glove):
    req: Request = create_request(headers={'x-forwarded-for': '09.155.161.152,1.1.1.1,162.158.90.14'})
    m = CloudflareCheckMiddleware(create_request.app)

    assert m.ip_ranges is None

    r = await m.dispatch(req, next_function)
    assert r.status_code == 200, r.body

    assert isinstance(m.ip_ranges, list)
    assert len(m.ip_ranges) == 22
    assert repr(m.ip_ranges[0]) == 'IPRangeCounter(162.158.0.0/15, 1)'


async def test_cloudflare_ok_client(create_request, glove):

    req: Request = create_request(client_addr='162.158.186.183')
    m = CloudflareCheckMiddleware(create_request.app)

    assert m.ip_ranges is None

    r = await m.dispatch(req, next_function)
    assert r.status_code == 200, r.body

    assert isinstance(m.ip_ranges, list)
    assert repr(m.ip_ranges[0]) == 'IPRangeCounter(162.158.0.0/15, 1)'


async def test_cloudflare_bad(create_request, glove):
    req: Request = create_request()
    m = CloudflareCheckMiddleware(create_request.app)

    assert m.ip_ranges is None

    r = await m.dispatch(req, next_function)
    assert r.status_code == 400, r.body
    assert r.body.startswith(b'Request incorrectly routed, this looks like')

    assert m.ip_ranges is None


async def test_cloudflare_bad2(create_request, glove):
    req: Request = create_request(client_addr='63.143.42.246')
    m = CloudflareCheckMiddleware(create_request.app, 'badness!')

    assert m.ip_ranges is None

    r = await m.dispatch(req, next_function)
    assert r.status_code == 400, r.body
    assert r.body == b'badness!'

    assert isinstance(m.ip_ranges, list)
    assert repr(m.ip_ranges[0]) == 'IPRangeCounter(173.245.48.0/20, 0)'


async def test_cloudflare_single_ip(create_request, glove):
    req: Request = create_request(headers={'x-forwarded-for': '1.1.1.1'})
    m = CloudflareCheckMiddleware(create_request.app)

    assert m.ip_ranges is None

    r = await m.dispatch(req, next_function)
    assert r.status_code == 400, r.body
    assert r.body.startswith(b'Request incorrectly routed, this looks like')

    assert isinstance(m.ip_ranges, list)
    assert len(m.ip_ranges) == 22


async def test_cloudflare_multiple(create_request, glove, mocker):
    get_cloudflare_ips_spy = mocker.spy(foxglove.middleware, 'get_cloudflare_ips')

    m = CloudflareCheckMiddleware(create_request.app)
    assert m.ip_ranges is None

    for client_ip in '162.158.186.183', '104.16.0.0', '162.158.92.59':
        req: Request = create_request(client_addr=client_ip)
        r = await m.dispatch(req, next_function)
        assert r.status_code == 200, r.body

    assert isinstance(m.ip_ranges, list)
    assert repr(m.ip_ranges[0]) == 'IPRangeCounter(162.158.0.0/15, 2)'
    assert repr(m.ip_ranges[1]) == 'IPRangeCounter(104.16.0.0/13, 1)'
    assert repr(m.ip_ranges[2]) == 'IPRangeCounter(173.245.48.0/20, 0)'
    assert get_cloudflare_ips_spy.call_count == 1


def test_index(client: Client):
    assert client.post_json('/no-csrf/') is None
    assert client.last_response.status_code == 200


@pytest.mark.parametrize(
    'headers,result',
    [
        ({}, 'Missing Origin and Referrer headers'),
        ({'foo': '1234'}, 'Missing Origin and Referrer headers'),
        ({'referer': 'https://www.example.com/foo/bar'}, None),
        ({'referer': 'http://www.example.org/foo/bar'}, None),
        ({'origin': 'https://www.example.com'}, None),
        ({'origin': 'http://www.example.org'}, None),
        ({'referer': 'https://www.different.com/foo/bar'}, 'Incorrect Referrer header'),
        ({'origin': 'https://www.example.com:8000'}, 'Incorrect Origin header'),
        ({'origin': 'https://www.different.com'}, 'Incorrect Origin header'),
        ({'origin': 'https://www.example.com', 'referer': 'https://www.example.com/foo/bar'}, None),
    ],
)
async def test_csrf_header_check(create_request, headers, result):
    m = CsrfMiddleware(
        create_request.app,
        enable_header_check=True,
        allows_origins={'https://www.example.com', 'http://www.example.org'},
    )

    req: Request = create_request(method='POST', headers=headers)
    assert m.header_check(req) == result


async def test_csrf_header_check_error(create_request):
    m = CsrfMiddleware(create_request.app, enable_header_check=True, allows_origins={'https://www.example.com'})

    req: Request = create_request(method='POST', session={'session_id': 'foobar'})
    r = await m.dispatch(req, next_function)
    assert r.status_code == 403, r.body
    assert json.loads(r.body) == {'message': 'Permission Denied, Missing Origin and Referrer headers'}



import logging

import pytest

from foxglove import exceptions
from foxglove.recaptcha import check_recaptcha
from foxglove.test_server import DummyServer
from foxglove.testing import TestClient as Client


def test_success(client: Client, settings, dummy_server: DummyServer, caplog):
    caplog.set_level(logging.INFO)
    settings.recaptcha_url = f'{dummy_server.server_name}/recaptcha_url/'
    assert client.get_json('/') == {'app': 'foxglove-demo'}
    assert client.post_json('/captcha-check/', {'recaptcha_token': '__ok__'}) == {'status': 'ok'}
    assert dummy_server.log == ['POST /recaptcha_url/ > 200 (recaptcha __ok__)']
    logs = [r.message for r in caplog.records if r.name == 'foxglove.recaptcha']
    assert logs == ['recaptcha success']


def test_no_token(client: Client, settings, dummy_server: DummyServer):
    settings.recaptcha_url = f'{dummy_server.server_name}/recaptcha_url/'
    assert client.get_json('/') == {'app': 'foxglove-demo'}
    assert client.post_json('/captcha-check/', {}, status=400) == {'message': 'No recaptcha value'}
    assert dummy_server.log == []


def test_wrong_host(client: Client, settings, dummy_server: DummyServer, caplog):
    caplog.set_level(logging.INFO)
    settings.recaptcha_url = f'{dummy_server.server_name}/recaptcha_url/'
    assert client.get_json('/') == {'app': 'foxglove-demo'}
    assert client.post_json('/captcha-check/', {'recaptcha_token': '__wrong_host__'}, status=400) == {
        'message': 'Invalid recaptcha value'
    }
    assert dummy_server.log == ['POST /recaptcha_url/ > 200 (recaptcha __wrong_host__)']
    logs = [r.message for r in caplog.records if r.name == 'foxglove.recaptcha']
    assert len(logs) == 1
    assert logs[0] == (
        'recaptcha failure, path=/captcha-check/ allowed_hosts=testserver ip=testclient '
        'response={"success": true, "hostname": "__wrong_host__"}'
    )


def test_bad_token(client: Client, settings, dummy_server: DummyServer, caplog):
    caplog.set_level(logging.INFO)
    settings.recaptcha_url = f'{dummy_server.server_name}/recaptcha_url/'
    assert client.get_json('/') == {'app': 'foxglove-demo'}
    assert client.post_json('/captcha-check/', {'recaptcha_token': 'bad'}, status=400) == {
        'message': 'Invalid recaptcha value'
    }
    assert dummy_server.log == ['POST /recaptcha_url/ > 200 (recaptcha bad)']
    logs = [r.message for r in caplog.records if r.name == 'foxglove.recaptcha']
    assert len(logs) == 1
    assert logs[0] == (
        'recaptcha failure, path=/captcha-check/ allowed_hosts=testserver ip=testclient '
        'response={"success": false, "hostname": "testserver"}'
    )


def test_settings_origin(client: Client, settings, dummy_server: DummyServer, caplog):
    caplog.set_level(logging.INFO)
    settings.recaptcha_url = f'{dummy_server.server_name}/recaptcha_url/'
    settings.origin = 'https://example.com'
    try:
        assert client.get_json('/') == {'app': 'foxglove-demo'}
        assert client.post_json('/captcha-check/', {'recaptcha_token': '__ok__ host:example.com'}) == {'status': 'ok'}
        assert dummy_server.log == ['POST /recaptcha_url/ > 200 (recaptcha __ok__ host:example.com)']
        logs = [r.message for r in caplog.records if r.name == 'foxglove.recaptcha']
        assert logs == ['recaptcha success']
    finally:
        settings.origin = None


async def test_direct_ok(create_request, settings, dummy_server: DummyServer, glove):
    settings.recaptcha_url = f'{dummy_server.server_name}/recaptcha_url/'
    await check_recaptcha(create_request(), '__ok__')


async def test_direct_request_origin(create_request, settings, dummy_server: DummyServer, glove):
    settings.recaptcha_url = f'{dummy_server.server_name}/recaptcha_url/'
    await check_recaptcha(create_request(headers={'origin': 'https://foo.com'}), '__ok__ host:foo.com')


async def test_direct_wrong_host(create_request, settings, dummy_server: DummyServer, glove):
    settings.recaptcha_url = f'{dummy_server.server_name}/recaptcha_url/'
    with pytest.raises(exceptions.HttpBadRequest):
        await check_recaptcha(create_request(), '__ok__ host:foobar.com')


async def test_allowed_hosts(create_request, settings, dummy_server: DummyServer, glove):
    settings.recaptcha_url = f'{dummy_server.server_name}/recaptcha_url/'
    await check_recaptcha(create_request(), '__ok__ host:foobar.com', allowed_hosts={'foobar.com'})



import pytest

from foxglove.utils import dict_not_none, list_not_none


def test_list_not_none():
    assert list_not_none(1, 2, 3) == [1, 2, 3]
    assert list_not_none(1, 2, None, 3) == [1, 2, 3]
    assert list_not_none(1, 2, False, 3) == [1, 2, False, 3]


def test_dict_not_none():
    assert dict_not_none({'a': 1, 'b': None, 'c': 3}) == {'a': 1, 'c': 3}
    assert dict_not_none(a=1, c=3) == {'a': 1, 'c': 3}
    with pytest.raises(TypeError, match='dict_not_none expected at most 1 argument, got 2'):
        dict_not_none({'a': 1}, {'b': None})
    with pytest.raises(TypeError, match='dict_not_none must be a dict, got list'):
        dict_not_none([1])



import asyncio
import os

import pytest

from foxglove.test_server import DummyServer, Offline


async def test_dummy_get(dummy_server: DummyServer, glove):
    r = await glove.http.get(f'http://localhost:{dummy_server.server.port}/status/200/')
    assert r.status_code == 200, r.text
    assert dummy_server.log == ['GET /status/200/ > 200']


async def test_recaptcha_dummy(dummy_server: DummyServer, glove):
    r = await glove.http.post(
        f'http://localhost:{dummy_server.server.port}/recaptcha_url/', data={'response': '__ok__'}
    )
    assert r.status_code == 200, r.text
    assert r.json() == {'success': True, 'hostname': 'testserver'}
    assert dummy_server.log == ['POST /recaptcha_url/ > 200 (recaptcha __ok__)']


async def test_recaptcha_dummy_400(dummy_server: DummyServer, glove):
    r = await glove.http.post(
        f'http://localhost:{dummy_server.server.port}/recaptcha_url/', data={'response': '__400__'}
    )
    assert r.status_code == 400, r.text
    assert r.json() == {}
    assert dummy_server.log == ['POST /recaptcha_url/ > 400 (recaptcha __400__)']


async def test_recaptcha_dummy_other(dummy_server: DummyServer, glove):
    r = await glove.http.post(f'http://localhost:{dummy_server.server.port}/recaptcha_url/')
    assert r.status_code == 200, r.text
    assert r.json() == {'success': False, 'hostname': 'testserver'}
    assert dummy_server.log == ['POST /recaptcha_url/ > 200 (recaptcha None)']


async def test_dummy_405(dummy_server: DummyServer, glove):
    r = await glove.http.get(f'http://localhost:{dummy_server.server.port}/recaptcha_url/')
    assert r.status_code == 405, r.text
    assert dummy_server.log == ['GET /recaptcha_url/ > 405']


def test_offline_online():
    o = Offline()
    assert bool(o) is False
    assert bool(o) is False


def test_offline_offline(mocker, capsys):
    ci_value = os.getenv('CI')
    if ci_value:
        del os.environ['CI']
    try:
        m = mocker.patch('aiodns.DNSResolver.query', side_effect=asyncio.TimeoutError)
        o = Offline()
        assert bool(o) is True
        assert bool(o) is True
        m.assert_called_once()
        captured = capsys.readouterr()
        assert captured.out == ''
        assert captured.err == '\nnot online: TimeoutError \n\n'
    finally:
        if ci_value:
            os.environ['CI'] = ci_value


_offline = Offline()
skip_if_offline = pytest.mark.skipif(_offline, reason='not online')


@skip_if_offline
def test_offline_decorator():
    print("we're online!")



import pytest
from dirty_equals import IsStr

from foxglove.testing import TestClient as Client


def test_template(client: Client):
    r = client.get('/template/')
    assert r.status_code == 200, r.text
    assert r.headers['content-type'] == 'text/html; charset=utf-8'
    assert r.text == '<p>Hello Samuel</p>'


def test_template_status(client: Client):
    r = client.get('/template/456/')
    assert r.status_code == 456, r.text
    assert r.headers['content-type'] == 'text/html; charset=utf-8'
    assert r.text == '<p>Hello ???</p>'


def test_template_static_url(client: Client):
    r = client.get('/template/spam/')
    assert r.status_code == 200, r.text
    assert r.headers['content-type'] == 'text/html; charset=utf-8'
    assert r.text == IsStr(regex=r'<script src="http://testserver/static/foobar\.js\?v=[0-9a-f]{7}"></script>')


def test_no_request(glove):
    from foxglove.templates import FoxgloveTemplates

    templates = FoxgloveTemplates()
    with pytest.raises(ValueError, match='context must include a "request" key'):
        templates.TemplateResponse('foobar', {})



import pytest
from dirty_equals import IsBytes
from pydantic import SecretBytes

from foxglove.auth import bcrypt as auth_bcrypt, check_password_breached, check_password_correct, hash_password
from foxglove.exceptions import HttpUnprocessableEntity
from foxglove.testing import TestClient as Client


async def test_password_hash(settings):
    pw_hash = await hash_password(SecretBytes(b'testing'))

    assert pw_hash.startswith('$2b$04$')


async def test_password_correct(settings, mocker):
    check_pw = mocker.spy(auth_bcrypt, 'checkpw')
    pw_hash = await hash_password(SecretBytes(b'testing'))
    assert await check_password_correct(SecretBytes(b'testing'), pw_hash) is True
    assert check_pw.call_args.args[0] == b'testing'
    assert check_pw.call_args.args[1].startswith(b'$2b$04$')
    assert check_pw.spy_return is True


async def test_password_correct_null(settings):
    pw_hash = await hash_password(SecretBytes(b'testing'))
    assert await check_password_correct(SecretBytes(b'test\x00ing'), pw_hash) is False


async def test_password_none(settings, mocker):
    check_pw = mocker.spy(auth_bcrypt, 'checkpw')
    assert await check_password_correct(SecretBytes(b'testing'), None) is False
    assert check_pw.call_args.args[0] == b'testing'
    assert check_pw.call_args.args[1].startswith(b'$2b$04$')


@pytest.mark.parametrize(
    'password,threshold,okay',
    [
        ('password', 100, False),
        ('foobar123', 2000, True),
        ('foobar123', 100, False),
        ('this-is-not-a-known-password-8dke03m4', 0, True),
    ],
)
async def test_check_password_breached(glove, password, threshold, okay):
    if okay:
        await check_password_breached(SecretBytes(password.encode()), threshold)
    else:
        with pytest.raises(HttpUnprocessableEntity, match='This password is known to hackers'):
            await check_password_breached(SecretBytes(password.encode()), threshold)


def get_redis_keys(glove, loop):
    async def _run():
        keys = await glove.redis.keys('rate-limit*')
        if not keys:
            return None
        else:
            assert len(keys) == 1
            key = keys[0]
            return key, await glove.redis.get(key)

    return loop.run_until_complete(_run())


def test_rate_limit_error(client: Client, glove, loop):
    assert get_redis_keys(glove, loop) is None
    assert client.get_json('/rate-limit-error/') == 'ok'
    assert get_redis_keys(glove, loop) == (IsBytes(regex=br'rate-limit:GET/rate-limit-error/:testclient:\d+'), b'1')
    assert client.get_json('/rate-limit-error/?foo=bar') == 'ok'
    assert get_redis_keys(glove, loop) == (IsBytes(regex=br'rate-limit:GET/rate-limit-error/:testclient:\d+'), b'2')
    assert client.get_json('/rate-limit-error/?spam=different', status=429) == {
        'message': 'rate limit of 2 requests per 1000 seconds exceeded'
    }
    assert get_redis_keys(glove, loop) == (IsBytes(regex=br'rate-limit:GET/rate-limit-error/:testclient:\d+'), b'3')
    assert client.get_json('/rate-limit-error/', status=429) == {
        'message': 'rate limit of 2 requests per 1000 seconds exceeded'
    }
    assert get_redis_keys(glove, loop) == (IsBytes(regex=br'rate-limit:GET/rate-limit-error/:testclient:\d+'), b'4')


def test_rate_limit_return(client: Client, glove, loop):
    assert get_redis_keys(glove, loop) is None
    assert client.get_json('/rate-limit-return/') == {'request_count': 1}
    assert client.get_json('/rate-limit-return/') == {'request_count': 2}
    assert get_redis_keys(glove, loop) == (IsBytes(regex=br'rate-limit:GET/rate-limit-return/:testclient:\d+'), b'2')



from typing import Dict

from buildpg.asyncpg import BuildPgConnection

from foxglove import glove
from foxglove.db.patches import patch, run_sql_section


@patch
async def check_args(conn: BuildPgConnection, args: Dict[str, str], logger, **kwargs):
    """
    check args are working right
    """
    logger.info('checking args: %s', args)
    assert args == {'user_id': '123', 'co': 'Testing'}
    assert await conn.fetchval('select 4^4') == 256


@patch
async def insert_org(conn: BuildPgConnection, args: Dict[str, str], **kwargs):
    """
    add an org, optionally fail
    """
    await conn.execute('delete from organisations')
    await conn.execute("insert into organisations (name) values ('foobar')")
    if 'fail' in args:
        raise RuntimeError('deliberate error')


@patch(auto_run=True, auto_sql_section='full_name')
async def run_full_name(conn: BuildPgConnection, args: Dict[str, str], **kwargs):
    """
    example of migrations patch
    """
    await run_sql_section('full_name', glove.settings.sql, conn)



from foxglove.testing import TestClient as Client


def test_index(client: Client):
    assert client.last_response is None
    assert client.get_json('/') == {'app': 'foxglove-demo'}
    assert client.last_response.status_code == 200


def test_post_index(client: Client):
    client.get_json('/')
    assert client.post_json('/', status=None)
    assert client.last_response.status_code == 405


def test_request_error(client: Client, mocker):
    fail_mock = mocker.patch('foxglove.testing.pytest.fail')
    client.post_json('/')
    fail_mock.called_once()


def test_create_user(client: Client):
    client.get_json('/')
    assert client.post_json('/create-user/', {'first_name': 'Samuel', 'last_name': 'Colvin'}, status=201) == {
        'id': 123,
        'v': 16,
    }


def test_no_session_id(client: Client):
    assert client.post_json('/create-user/', {'first_name': 'Samuel', 'last_name': 'Colvin'}, status=403) == {
        'message': 'Permission Denied, no session set, updates not permitted'
    }


def test_errors_raise_unexpected(client: Client, caplog):
    assert client.get_json('/error/', status=400) == {'message': 'raised HttpBadRequest'}
    assert len(caplog.records) == 1, caplog.text
    assert '"GET /error/", unexpected response: 400' in caplog.text
    r = caplog.records[0]
    assert r.user == {'ip_address': 'testclient'}
    assert r.request['url'] == 'http://testserver/error/'
    assert r.extra['route_endpoint'] == 'error'
    assert r.extra['response_body'] == {'message': 'raised HttpBadRequest'}


def test_errors_exception(client: Client, caplog):
    r = client.get('/error/', params={'error': 'RuntimeError'})
    assert r.status_code == 500, r.text
    assert len(caplog.records) == 1, caplog.text
    assert '"GET /error/?error=RuntimeError", RuntimeError(' in caplog.text
    assert caplog.records[0].request['url'] == 'http://testserver/error/?error=RuntimeError'


def test_errors_request_return_unexpected(client: Client, caplog, mocker):
    m = mocker.patch('foxglove.middleware.capture_event')
    assert client.get_json('/error/', params={'error': 'return'}, status=400) == {'error': 'return'}
    assert len(caplog.records) == 1, caplog.text
    assert '"GET /error/?error=return", unexpected response: 400' in caplog.text
    r = caplog.records[0]
    assert r.user == {'ip_address': 'testclient'}
    assert r.request['url'] == 'http://testserver/error/?error=return'
    assert r.extra['route_endpoint'] == 'error'
    assert r.extra['response_body'] == {'error': 'return'}
    assert m.call_count == 0


def test_errors_expected_sentry(client_sentry: Client, caplog, mocker):
    m = mocker.patch('foxglove.middleware.capture_event')
    assert client_sentry.get_json('/error/', status=400) == {'message': 'raised HttpBadRequest'}
    assert len(caplog.records) == 1, caplog.text
    msg = '"GET /error/", unexpected response: 400'
    assert msg in caplog.text

    m.assert_called_once()
    assert m.call_args.args[0]['message'] == msg
    assert m.call_args.args[0]['fingerprint'] == ('/error/', 'GET', '400')
    assert m.call_args.args[1] is None


def test_errors_exception_sentry(client_sentry: Client, caplog, mocker):
    m = mocker.patch('foxglove.middleware.capture_event')
    r = client_sentry.get('/error/', params={'error': 'RuntimeError'})
    assert r.status_code == 500, r.text
    assert len(caplog.records) == 1, caplog.text
    msg = '"GET /error/?error=RuntimeError", RuntimeError(\'raised RuntimeError\')'
    assert msg in caplog.text

    m.assert_called_once()
    assert m.call_args.args[0]['message'] == msg
    assert m.call_args.args[0]['fingerprint'] == ('/error/', 'GET', "RuntimeError('raised RuntimeError')")
    assert m.call_args.args[1] is not None


def test_null_json_error(client: Client):
    client.get_json('/')
    assert client.post_json('/create-user/', {'first_name': 'Samuel\x00', 'last_name': 'Colvin'}, status=400) == {
        'detail': 'There was an error parsing the body'
    }



from buildpg.asyncpg import BuildPgConnection
from starlette.requests import Request as StarletteRequest
from starlette.responses import JSONResponse
from starlette.routing import Route

from foxglove import BaseSettings, glove


class FoxGloveState:
    conn: BuildPgConnection


class Request(StarletteRequest):
    state: FoxGloveState


async def homepage(request: Request):
    v = await request.state.conn.fetchval('select 4^4')
    r = await glove.http.get('https://www.example.com')
    return JSONResponse({'hello': v, 'response_status': r.status_code})


routes = [Route('/', homepage)]


class Settings(BaseSettings):
    pass



import asyncio
import logging
from typing import Optional

from buildpg.asyncpg import BuildPgConnection
from fastapi import Depends, FastAPI, Request
from fastapi.staticfiles import StaticFiles
from pydantic import BaseModel
from starlette.middleware import Middleware
from starlette.middleware.sessions import SessionMiddleware

from foxglove import BaseSettings, exceptions, glove
from foxglove.auth import rate_limit
from foxglove.db import PgMiddleware
from foxglove.db.middleware import get_db
from foxglove.middleware import CsrfMiddleware, ErrorMiddleware
from foxglove.recaptcha import RecaptchaDepends
from foxglove.route_class import SafeAPIRoute
from foxglove.templates import FoxgloveTemplates

logger = logging.getLogger('main')


# first middleware is outer here
middleware = []


def add_middleware(mw: type, **kwargs):
    middleware.append(Middleware(mw, **kwargs))


add_middleware(ErrorMiddleware)
add_middleware(
    SessionMiddleware,
    secret_key=glove.settings.secret_key,
    session_cookie=glove.settings.cookie_name,
    same_site='strict',
)


def should_check_csrf(request):
    return request.url.path != '/no-csrf/'


add_middleware(CsrfMiddleware, should_check=should_check_csrf)
add_middleware(PgMiddleware)


app = FastAPI(
    title='Foxglove Testing',
    on_startup=[glove.startup],
    on_shutdown=[glove.shutdown],
    middleware=middleware,
    docs_url=None,
    redoc_url='/docs',
)
app.router.route_class = SafeAPIRoute
app.mount('/static', StaticFiles(directory=glove.settings.static), name='static')


@app.exception_handler(exceptions.HttpMessageError)
async def foxglove_exception_handler(request: Request, exc: exceptions.HttpMessageError):
    return exceptions.HttpMessageError.handle(exc)


@app.get('/')
async def index(request: Request):
    return {'app': 'foxglove-demo'}


class UserInfo(BaseModel):
    first_name: str
    last_name: str
    email: str = None


@app.post('/create-user/', status_code=201)
async def create_user(user: UserInfo, conn: BuildPgConnection = Depends(get_db)):
    logger.info('user: %s', user)
    v = await conn.fetchval('select 4^2')
    return {'id': 123, 'v': v}


@app.get('/error/', status_code=400)
async def error(error: str = 'raise'):
    if error == 'RuntimeError':
        raise RuntimeError('raised RuntimeError')
    elif error == 'raise':
        raise exceptions.HttpBadRequest('raised HttpBadRequest')
    else:
        return {'error': error}


def worker(settings: BaseSettings):
    asyncio.run(aworker(settings))


async def aworker(settings: BaseSettings):
    async with glove.context():

        async with glove.pg.acquire() as conn:
            v = await conn.fetchval('select 4^4')
        logger.info('running demo worker function, ans: %0.0f', v)
        assert isinstance(settings, BaseSettings)


class CheckRecaptchaModal(BaseModel):
    recaptcha_token: Optional[str]


@app.post('/captcha-check/')
async def captcha_check(m: CheckRecaptchaModal, check_recaptcha: RecaptchaDepends = Depends(RecaptchaDepends)):
    await check_recaptcha(m.recaptcha_token)
    return {'status': 'ok'}


@app.post('/no-csrf/')
async def no_csrf():
    pass


@app.get('/rate-limit-error/', dependencies=[Depends(rate_limit(request_limit=2, interval=1000))])
async def rate_limit_raise():
    return 'ok'


@app.get('/rate-limit-return/')
async def rate_limit_return(request_count: int = Depends(rate_limit(request_limit=None, interval=1000))):
    return {'request_count': request_count}


templates = FoxgloveTemplates()


@app.get('/template/')
@templates.render('foobar.jinja')
async def render_template(request: Request):
    return {'name': 'Samuel'}


@app.get('/template/456/')
@templates.render('foobar.jinja')
def render_template_456(request: Request):
    return 456, None


@app.get('/template/spam/')
@templates.render('spam.jinja')
def render_template_spam(request: Request):
    pass



from typer.testing import CliRunner

from foxglove import glove
from foxglove.cli import cli


def test_print_commands():
    runner = CliRunner()
    result = runner.invoke(cli, ['--help'])
    assert result.exit_code == 0, result.output
    assert 'foxglove command line interface' in result.output
    assert 'Run the web server using uvicorn for development' in result.output


def test_patches_print():
    runner = CliRunner()
    result = runner.invoke(cli, ['-s', 'demo.settings', 'patch'])
    assert result.exit_code == 0, result.output
    assert 'rerun_sql: rerun the contents of settings.sql_path' in result.output
    assert 'check_args: check args are working right' in result.output


def test_patches_check_args(clean_db, loop):
    runner = CliRunner()
    result = runner.invoke(cli, ['-s', 'demo.settings', 'patch', 'check_args', '-a', 'user_id:123', '-a', 'co:Testing'])
    assert result.exit_code == 0, result.output
    assert "checking args: {'user_id': '123', 'co': 'Testing'}" in result.output


def test_web_no_settings(mocker):
    mock_uvicorn_run = mocker.patch('foxglove.cli.uvicorn_run')
    runner = CliRunner()
    result = runner.invoke(cli, ['web'])
    assert result.exit_code == 1, result.output
    assert 'unable to infer settings path' in result.output
    assert mock_uvicorn_run.call_count == 0


def test_web(mocker):
    mock_uvicorn_run = mocker.patch('foxglove.cli.uvicorn_run')
    runner = CliRunner()
    result = runner.invoke(cli, ['-s', 'demo.settings', 'web'])
    assert result.exit_code == 0, result.output
    assert 'running web server at 8000...' in result.output
    mock_uvicorn_run.assert_called_once()
    assert mock_uvicorn_run.call_args.kwargs['host'] == '0.0.0.0'
    assert mock_uvicorn_run.call_args.kwargs['port'] == 8000
    assert mock_uvicorn_run.call_args.kwargs.get('reload') is None
    assert mock_uvicorn_run.call_args.kwargs.get('debug') is None
    assert mock_uvicorn_run.call_args.kwargs.get('access_log') is False


def test_dev(mocker):
    mock_uvicorn_run = mocker.patch('foxglove.cli.uvicorn_run')
    runner = CliRunner()
    result = runner.invoke(cli, ['-s', 'demo.settings:Settings', 'dev'])
    assert result.exit_code == 0, result.output
    assert 'running web server at 8000 in dev mode...' in result.output
    mock_uvicorn_run.assert_called_once()
    assert mock_uvicorn_run.call_args.kwargs['host'] == '127.0.0.1'
    assert mock_uvicorn_run.call_args.kwargs['port'] == 8000
    assert mock_uvicorn_run.call_args.kwargs.get('reload') is True
    assert mock_uvicorn_run.call_args.kwargs.get('debug') is True
    assert mock_uvicorn_run.call_args.kwargs.get('access_log') is None


def test_auto_web(mocker):
    mock_uvicorn_run = mocker.patch('foxglove.cli.uvicorn_run')
    runner = CliRunner()
    result = runner.invoke(cli, ['-s', 'demo.settings', 'auto'], env={'FOXGLOVE_COMMAND': 'web'})
    assert result.exit_code == 0, result.output
    assert 'running web server at 8000...' in result.output
    assert mock_uvicorn_run.call_count == 1
    assert mock_uvicorn_run.call_args.kwargs['port'] == 8000

    del glove._settings
    result = runner.invoke(cli, ['-s', 'demo.settings', 'auto'], env={'DYNO': 'WEB.1'})
    assert result.exit_code == 0, result.output
    assert 'running web server at 8000...' in result.output
    assert mock_uvicorn_run.call_count == 2
    assert mock_uvicorn_run.call_args.kwargs['port'] == 8000

    del glove._settings
    result = runner.invoke(cli, ['-s', 'demo.settings', 'auto'], env={'PORT': '5000'})
    assert result.exit_code == 0, result.output
    assert 'running web server at 5000...' in result.output
    assert mock_uvicorn_run.call_count == 3
    assert mock_uvicorn_run.call_args.kwargs['port'] == 5000


def test_worker(loop):
    runner = CliRunner()
    result = runner.invoke(cli, ['-s', 'demo.settings', 'worker'])
    assert result.exit_code == 0, result.output
    assert 'running worker...' in result.output
    assert 'running demo worker function, ans: 256' in result.output


def test_reset_database(mocker):
    mock_uvicorn_run = mocker.patch('foxglove.db.reset_database')
    runner = CliRunner()
    result = runner.invoke(cli, ['-s', 'demo.settings', 'reset_database'])
    assert 'running reset_database...' in result.output
    assert mock_uvicorn_run.call_count == 1


def test_run_migrations(mocker):
    mock_uvicorn_run = mocker.patch('foxglove.db.migrations.run_migrations')
    runner = CliRunner()
    result = runner.invoke(cli, ['-s', 'demo.settings', 'migrations'])
    assert 'running migrations live=False fake=False...' in result.output
    assert mock_uvicorn_run.call_count == 1


def test_run_migrations_fake(mocker):
    mock_uvicorn_run = mocker.patch('foxglove.db.migrations.run_migrations')
    runner = CliRunner()
    result = runner.invoke(cli, ['-s', 'demo.settings', 'migrations', '--live', '--fake'])
    assert 'running migrations live=True fake=True...' in result.output
    assert mock_uvicorn_run.call_count == 1



import sys

sys.stderr.write(
    """
===============================
Unsupported installation method
===============================
foxglove-web no longer supports installation with `python setup.py install`.
Please use `python -m pip install .` instead.
"""
)
sys.exit(1)


# The below code will never execute, however GitHub is particularly
# picky about where it finds Python packaging metadata.
# See: https://github.com/github/feedback/discussions/6456
#
# To be removed once GitHub catches up.

setup(
    name='foxglove-web',
    install_requires=[
        'arq>=0.23',
        'asyncpg>=0.23.0',
        'fastapi>=0.72',
        'itsdangerous>=1.1.0',
        'buildpg>=0.3.0',
        'httpx>=0.21.1',
        'pydantic>=1.8.2',
        'sentry-sdk>=1',
        'typer>=0.3.2',
        'uvicorn>=0.13.3',
    ],
)



import logging

from buildpg.asyncpg import BuildPgConnection
from dirty_equals import IsNow, IsPositiveInt

from foxglove.db import prepare_database
from foxglove.db.utils import AsyncPgContext
from foxglove.redis import async_flush_redis, flush_redis
from foxglove.settings import BaseSettings
from tests.conftest import ConnContext


async def test_prepare_database(db_conn_global: BuildPgConnection, alt_settings: BaseSettings, caplog):
    await db_conn_global.execute(f'drop database if exists {alt_settings.pg_name}')
    alt_settings.pg_migrations = True

    assert (
        await db_conn_global.fetchval('SELECT true FROM pg_catalog.pg_database where datname=$1', alt_settings.pg_name)
        is None
    )

    with caplog.at_level(logging.INFO, 'foxglove.db'):
        assert await prepare_database(alt_settings, True) is True

    assert (
        await db_conn_global.fetchval('SELECT true FROM pg_catalog.pg_database where datname=$1', alt_settings.pg_name)
        is True
    )

    with caplog.at_level(logging.INFO, 'foxglove'):
        assert await prepare_database(alt_settings, False) is False

    async with AsyncPgContext(alt_settings.pg_dsn) as conn:
        assert await conn.fetchval('select count(*) from migrations') == 1
        migrations = dict(await conn.fetchrow('select * from migrations'))

    assert migrations == {
        'id': IsPositiveInt,
        'ref': 'run_full_name',
        'sql_section': (
            'full_name::\n'
            'create or replace function full_name(u users) returns varchar as $$\n'
            '  begin\n'
            "    return coalesce(u.first_name || ' ' || u.last_name, u.first_name, u.last_name);\n"
            '  end;\n'
            '$$ language plpgsql;'
        ),
        'ts': IsNow(tz='utc'),
        'fake': True,
    }

    assert caplog.messages == [
        'database successfully setup ✓',
        'migrations table created',
        'checking 1 migration patches...',
        'faked migration run_full_name',
        '1 migration patches faked, 0 already up to date ✓',
        'database already exists ✓',
        'checking 1 migration patches...',
        'all 1 migrations already up to date ✓',
    ]
    alt_settings.pg_migrations = False


async def test_prepare_database_replace(alt_settings: BaseSettings):
    await prepare_database(alt_settings, True)

    async with ConnContext(alt_settings.pg_dsn) as conn:
        await conn.execute("insert into organisations (name) values ('foobar')")
        assert await conn.fetchval('select count(*) from organisations') == 1

    assert await prepare_database(alt_settings, True) is True

    async with ConnContext(alt_settings.pg_dsn) as conn:
        assert await conn.fetchval('select count(*) from organisations') == 0


async def test_prepare_database_keep(db_conn_global, alt_settings: BaseSettings):
    await prepare_database(alt_settings, True)

    async with ConnContext(alt_settings.pg_dsn) as conn:
        await conn.execute("insert into organisations (name) values ('foobar')")
        assert await conn.fetchval('select count(*) from organisations') == 1

    assert await prepare_database(alt_settings, False) is False

    async with ConnContext(alt_settings.pg_dsn) as conn:
        assert await conn.fetchval('select count(*) from organisations') == 1


def test_flush_redis_yes(settings, caplog, mocker):
    caplog.set_level(logging.INFO)
    mocker.patch('foxglove.redis.input', return_value='y')
    flush_redis(settings)
    assert caplog.record_tuples == [
        ('foxglove.redis', logging.INFO, 'resetting database...'),
        ('foxglove.redis', logging.INFO, 'done.'),
    ]


def test_flush_redis_no(settings, caplog, mocker):
    caplog.set_level(logging.INFO)
    mocker.patch('foxglove.redis.input', return_value='n')
    flush_redis(settings)
    assert caplog.record_tuples == [('foxglove.redis', logging.INFO, 'cancelling')]


async def test_async_flush_redis(settings, glove):
    await glove.redis.set('foo', '1')
    await glove.redis.set('bar', '2')
    assert len(await glove.redis.keys('*')) == 2
    await async_flush_redis(settings)
    assert len(await glove.redis.keys('*')) == 0
